!function($,D,window,document,undefined) {
  "use strict";

  Drupal.behaviors.dismissible_message_bar = {
    attach: function (context, settings) {

      /**
       * Retrieve DMB Notification Settings.
       *
       * @return {Array}
       *   Array of defined DMB Settings.
       */
      const getNotificationSettings = function () {
        if (typeof(settings.dmbNotifications) !== 'undefined' && settings.dmbNotifications) {
          return settings.dmbNotifications;
        }
        return null;
      };

      /**
       * Inject Notifications into their respective wrappers.
       */
      const injectNotifications = function (notificationSettings, $notificationWrapper, requestData) {

        let pathObj = drupalSettings.path,
          requestPath = pathObj.baseUrl + pathObj.pathPrefix + "ajax/dmb_notifications";

        if (notificationSettings !== null) {
          $.ajax({
            type: "POST",
            url: requestPath,
            data: requestData,
            success: function (data, text_status) {

              // If there are no notifications - remove their respective placeholders.
              if (typeof(data.notifications) === 'undefined' && $notificationWrapper !== null) {
                $notificationWrapper.remove();
              }

              for (let key in data) {
                // Skip loop if the property is from prototype
                if (!data.hasOwnProperty(key)) continue;

                // Inject notifications.
                if ($notificationWrapper !== null) {
                  if (key === 'notifications' && $notificationWrapper.length > 0) {
                    $notificationWrapper.once('inject-dmb-notifications').append(data[key]);
                    $notificationWrapper.once('inject-dmb-notifications-trigger').trigger('injectDMBNotifications');
                  }
                }

              }
            },
          });
        }
      };

      // Initialize notifications.
      let settingsObject = getNotificationSettings();
      let notificationSettingsArray = Object.values(settingsObject);
      for (let i in notificationSettingsArray) {
        let notificationSettings = notificationSettingsArray[i];
        let $notificationWrapper = null,
          $overlayWrapper = null,
          requestData = {
            contentType: '',
            contentPath: '',
            notificationType: '',
          };

        if (typeof(notificationSettings.notificationWrapper) !== 'undefined' && notificationSettings.notificationWrapper) {
          $notificationWrapper = $(notificationSettings.notificationWrapper);
        }

        if (typeof(notificationSettings.overlayWrapper) !== 'undefined' && notificationSettings.overlayWrapper) {
          $overlayWrapper = $(notificationSettings.overlayWrapper);
        }

        // These parameters have to be saved and passed into the handler.
        if (typeof(notificationSettings.contentType) !== 'undefined' && notificationSettings.contentType) {
          requestData.contentType = notificationSettings.contentType;
        }

        if (typeof(notificationSettings.contentPath) !== 'undefined' && notificationSettings.contentPath) {
          requestData.contentPath = notificationSettings.contentPath;
        }

        if (typeof(notificationSettings.notificationType) !== 'undefined' && notificationSettings.notificationType) {
          requestData.notificationType = notificationSettings.notificationType;
        }

        injectNotifications(notificationSettings, $notificationWrapper, requestData);

        if ($notificationWrapper !== null) {

          $notificationWrapper.on('injectDMBNotifications', function () {

            $('.dmb-notification .close-dmb-notification').once().on('click touchstart', function () {
              let $notification = $(this).parent(),
                notification_id = $notification.attr('dmb-notification-id'),
                accepted_notifications = $.cookie('dismissible_message_bar');

              if (!accepted_notifications) {
                accepted_notifications = notification_id;
              } else {
                accepted_notifications = accepted_notifications.split('.');
                if (!accepted_notifications.includes(notification_id)) {
                  accepted_notifications.push(notification_id);
                }
                accepted_notifications = accepted_notifications.join('.');
              }

              // Set the notification cookie with updated values
              const setAcceptedCookie = function() {
                $.cookie('dismissible_message_bar', accepted_notifications, {
                  expires: 365,
                  path: '/'
                });
              };

              setAcceptedCookie();

              $notification.slideUp();
              return false;
            });
          });
        }

      }

    }
  };

}(jQuery,Drupal,window,document,'');